#
# Ronin Exploits - A Ruby library for Ronin that provides exploitation and
# payload crafting functionality.
#
# Copyright (c) 2007-2009 Hal Brodigan (postmodern.mod3 at gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

require 'ronin/os'

require 'extlib'

module Ronin
  module Model
    module TargetsOS
      def self.included(base)
        base.module_eval do
          # The targeted OS
          belongs_to :os,
                     :model => 'Ronin::OS',
                     :nullable => true

          #
          # The targeted OS.
          #
          # @param [Array] arguments
          #   If any arguments are given, they will be used to create a new
          #   OS that will be targeted.
          #
          # @return [OS]
          #   The targeted OS.
          #
          # @example Accessing the targeted OS.
          #   target.os
          #   # => nil
          #
          # @example Setting the targeted OS.
          #   target.os(:name => 'FreeBSD', :version => '7.1')
          #   # => #<Ronin::OS type=Ronin::OS id=nil name="FreeBSD"
          #   # version="7.1">
          #
          def os(*arguments)
            unless arguments.empty?
              return self.os = OS.first_or_create(*arguments)
            else
              return relationships[:os].get(self)
            end
          end
        end

        model_name = base.name.split('::').last.snake_case.plural.to_sym
        OS.has OS.n, model_name, :model => base.name
      end
    end
  end
end
